function ABAEnderecoParceiroController(CRM_Utilities, templateManager,ScreenService, Messages, General, metaDataFactory, restEngine) {
    
    /**
     * Insere a widget de endereço na widget recebida como 
     * parâmetro.
     * 
     * @param {object} widget widget que receberá a a widget
     * de endereço("WDGT_ENDERECOS").
     * 
     * @param {string} nrparcnegocio nome referente ao
     * campo de parceiro de negócio.
    */

    this.setAddressWidgetOnActiveWidget = function(widget, position){
        
        widget.widgets = CRM_Utilities.setElementOnArrayDefinedPosition(this.factoryAddressWidget(widget), widget.widgets, position);
        widget.activate();
        
    };
    
    /**
     * Cria a widget dinâmica de endereços
     * na widget ativa recebida como parâmetro.
     * 
     * @param {object} activeWidget widget ativa.
    */

    this.factoryAddressWidget = function(activeWidget){
        
        var addressWidget = {
            "id":"1722319758311315390516",
            "name": "WDGT_ENDERECOS",
            "label": "Endereços",
            "template": "widget/master_detail/grid.html",
            "isVisible": true,
            "parent": activeWidget,
            "container": activeWidget.container,
            "dataSource": {
                "name": "product#/crud/enderecoparc_localizacao_tipoendereco",
                "rest": true
            },
            "paramsNewCode": {
                "__is_new_code": true,
                "__new_code_pk": "ENDERECOPARC",
                "__table_pk": "NRENDERECOPARC",
                "__cast_number": true
            },
            "dataSourceFilter":[
                {
                    "name": "NRPARCNEGOCIO",
                    "operator": "=",
                    "value": ""
                }
            ],
            "fieldGroups": [
                {
                    "name": "geolocalizacao",
                    "label": "Geolocalização",
                    "collapsible": true,
                    "opened": true
                }
            ],
            "events": [
                {
                    "name": "WidgetOnEnter",
                    "code": "ABAEnderecoParceiroController.setAddressWidgetFilters(args.owner);"
                },
                {
                    "name": "WidgetAfterMoveRow",
                    "code": "crm00040_Lead_Controller.validateMask(args.owner);"
                },
                {
                    "name": "WidgetOnSave",
                    "code": "args.owner.reload();"
                }
            ],
            "actions": [
                {
                    "name": "VOLTAR_LISTA",
                    "label": "Voltar",
                    "isVisible": true,
                    "showAsAction": "back_view",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "General.widgetGoBackToList(args.owner.widget);"
                        }
                    ]
                },
                {
                    "name": "VOLTAR_LISTA_PAI",
                    "label": "Voltar",
                    "isVisible": true,
                    "showAsAction": "back_list",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "General.widgetGoBackToParent(args.owner.widget);"
                        }
                    ]
                },
                {
                    "name": "ADICIONAR_ENDERECO",
                    "label": "Adicionar",
                    "icon": "plus",
                    "isVisible": true,
                    "showAsAction": "always|list",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "ABAEnderecoParceiroController.addAddress(args.owner.widget);"
                        }
                    ]
                },
                {
                    "name": "EDITAR_ENDERECO",
                    "label": "Editar",
                    "icon": "pencil",
                    "isVisible": true,
                    "showAsAction": "view",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "ABAEnderecoParceiroController.editAddress(args.owner.widget);"
                        }
                    ]
                },
                {
                    "name": "CANCELAR_FORMULARIO",
                    "label": "Cancelar",
                    "isVisible": true,
                    "showAsAction": "back_edit",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "General.widgetCancelForm(args.owner.widget);"
                        }
                    ]
                },
                {
                    "name": "SALVAR_ENDERECO",
                    "label": "Salvar",
                    "isVisible": true,
                    "showAsAction": "never_edit",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "CRM_Utilities.saveAndSetColumnToBeRetunrOnCurrentRow(args.owner.widget, args.owner.widget.paramsNewCode.__table_pk);"
                        }
                    ]
                },
                {
                    "name": "EXCLUIR_ENDERECO",
                    "label": "Excluir",
                    "isVisible": true,
                    "icon": "trash",
                    "showAsAction": "view",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "ABAEnderecoParceiroController.deleteAddress(args.owner.widget);"
                        }
                    ]
                },
                {
                    "name": "GEOCODEADRESS",
                    "label": "Buscar Latitude/Longitude",
                    "isVisible": true,
                    "showAsAction": "never_view",
                    "events": [
                        {
                            "name": "ActionEvent",
                            "code": "ABAEnderecoParceiroController.searchLocation();"
                        }
                    ]
                }
            ],
            "fields": [
                {
                    "name": "NMTIPOENDERECO",
                    "label": "Tipo de Endereço",
                    "class": "12",
                    "template": "field/select.html",
                    "isVisible": true,
                    "showOnList": true,
                    "showOnView": true,
                    "showOnForm": true,
                    "showSelectAs": "grid",
                    "descriptionOrder": 1,
                    "validations": {
                        "required": true
                    },
                    "dataSource": {
                        "name": "product#/crud/tipoendereco_unico_por_parceiro",
                        "rest": true,
                        "lazyLoad": true
                    },
                    "dataSourceFilter": [
                        {
                            "name": "P_NRPARCNEGOCIO",
                            "operator": "=",
                            "value": ""
                        },
                        {
                            "name": "P_CDTIPOENDERECO",
                            "operator": "=",
                            "value": ""
                        },
                        {
                            "name": "P_NRORG",
                            "operator": "=",
                            "value": ""
                        }
                    ],
                    "valueField": "CDTIPOENDERECO",
                    "descriptionField": "NMTIPOENDERECO",
                    "outData": {
                        "NMTIPOENDERECO": "NMTIPOENDERECO",
                        "CDTIPOENDERECO": "CDTIPOENDERECO"
                    },
                    "fields": [
                        {
                            "name": "NMTIPOENDERECO",
                            "label": "Tipo de Endereço",
                            "isVisible": true,
                            "showOnList": true,
                            "template": "field/text-edit.html"
                        }
                    ],
                    "events": [
                        {
                            "name": "FieldOnClick",
                            "code": "ABAEnderecoParceiroController.setAddressTypeFieldFilter(args.owner.field.widget);"
                        }
                    ],
                    "filterProperties": {
                        "events": [],
                        "validations": {}
                    }
                },
                {
                    "name": "IDATIVO",
                    "label": "Ativo",
                    "template": "field/select-native.html",
                    "class": "6",
                    "isVisible": true,
                    "showOnList": false,
                    "showOnView": true,
                    "showOnForm": true,
                    "defaultValue": "S",
                    "valueField": "value",
                    "descriptionField": "name",
                    "validations": {},
                    "hideEmptyValue": true,
                    "dataSource": {
                        "data": [
                            {
                                "name": "Sim",
                                "value": "S"
                            },
                            {
                                "name": "Não",
                                "value": "N"
                            }
                        ]
                    },
                    "filterProperties": {
                        "hideEmptyValue": false
                    }
                },
                {
                    "name": "NRCEPENDERECO",
                    "label": "CEP",
                    "template": "field/text-edit.html",
                    "class": "6",
                    "isVisible": true,
                    "showOnList": false,
                    "showOnView": true,
                    "showOnForm": true,
                    "validations": {
                        "required": true
                    },
                    "maxlength": 9,
                    "events": [
                        {
                            "name": "FieldOnChange",
                            "code": "ABAEnderecoParceiroController.fillAddressByCep(args.owner.field.widget);"
                        }
                    ],
                    "filterProperties": {
                        "events": [],
                        "validations": {},
                        "mask": {
                            "type": "fix",
                            "params": {}
                        }
                    }
                },
                {
                    "name": "DSENDERECO",
                    "label": "Endereço",
                    "template": "field/text-edit.html",
                    "class": "12",
                    "isVisible": true,
                    "showOnList": true,
                    "showOnView": true,
                    "showOnForm": true,
                    "validations": {},
                    "events": []
                },
                {
                    "name": "NRIMOVELENDERECO",
                    "label": "Número",
                    "template": "field/number-edit.html",
                    "class": "6",
                    "isVisible": true,
                    "showOnList": true,
                    "showOnView": true,
                    "showOnForm": true,
                    "minValue": 0,
                    "validations": {},
                    "events": []
                },
                {
                    "name": "DSCOMPLEENDERECO",
                    "label": "Complemento",
                    "template": "field/text-edit.html",
                    "class": "6",
                    "isVisible": true,
                    "showOnList": false,
                    "showOnView": true,
                    "showOnForm": true,
                    "validations": {},
                    "events": []
                },
                {
                    "name": "NMPAIS",
                    "label": "País",
                    "template": "field/select.html",
                    "class": "6",
                    "isVisible": true,
                    "showOnList": false,
                    "showOnView": true,
                    "showOnForm": true,
                    "defaultValue": "",
                    "validations": {
                        "required": true
                    },
                    "dataSource": {
                        "name": "product#/pais",
                        "lazyLoad": true,
                        "rest": true
                    },
                    "outData": {
                        "CDPAIS": "CDPAIS",
                        "NMPAIS": "NMPAIS"
                    },
                    "valueField": "CDPAIS",
                    "descriptionField": "NMPAIS",
                    "showSelectAs": "grid",
                    "fields": [
                        {
                            "name": "NMPAIS",
                            "label": "País",
                            "isVisible": true,
                            "showOnList": true,
                            "template": "field/text-edit.html"
                        }
                    ],
                    "events": [
                        {
                            "name": "FieldOnChange",
                            "code": "ABAEnderecoParceiroController.checkPlaceChange(args.owner.field);"
                        },
                        {
                            "name": "FieldOnClick",
                            "code": "CRM_Utilities.setDefaultOrgFilterAndOperatorOrgFilter(args.owner.field);"
                        }
                    ],
                    "filterProperties": {
                        "events": [],
                        "validations": {}
                    }
                },
                {
                    "name": "NMESTADO",
                    "label": "Estado",
                    "template": "field/select.html",
                    "class": "6",
                    "isVisible": true,
                    "showOnList": true,
                    "showOnView": true,
                    "showOnForm": true,
                    "showAddButtonAs": "button",
                    "addButtonText": "Adicionar Novo Estado",
                    "readOnly": true,
                    "validations": {
                        "required": true
                    },
                    "valueField": "SGESTADO",
                    "descriptionField": "NMESTADO",
                    "dataSource": {
                        "name": "product#/estado",
                        "rest": true
                    },
                    "dataSourceFilter": [],
                    "outData": {
                        "SGESTADO": "SGESTADO",
                        "NMESTADO": "NMESTADO"
                    },
                    "showSelectAs": "grid",
                    "fields": [
                        {
                            "name": "NMESTADO",
                            "label": "Estado",
                            "isVisible": true,
                            "showOnList": true,
                            "template": "field/text-edit.html"
                        }
                    ],
                    "events": [
                        {
                            "name": "FieldOnChange",
                            "code": "ABAEnderecoParceiroController.checkPlaceChange(args.owner.field);"
                        },
                        {
                            "id": "17223197582476052126733",
                            "name": "FieldOnAdd",
                            "code": "crm00040_Lead_Controller.criarNovoEstado(args.owner.field.widget)"
                        },
                        {
                            "name": "FieldOnClick",
                            "code": "ABAEnderecoParceiroController.addressFieldOnClick(args.owner.field);"
                        }
                    ],
                    "filterProperties": {
                        "events": [],
                        "validations": {},
                        "readOnly": false
                    }
                },
                {
                    "name": "NMMUNICIPIO",
                    "label": "Município",
                    "template": "field/select.html",
                    "class": "6",
                    "isVisible": true,
                    "showOnList": true,
                    "showOnView": true,
                    "showOnForm": true,
                    "readOnly": true,
                    "defaultValue": "",
                    "showAddButtonAs": "button",
                    "addButtonText": "Adicionar Novo Município",
                    "validations": {
                        "required": true
                    },
                    "valueField": "CDMUNICIPIO",
                    "descriptionField": "NMMUNICIPIO",
                    "dataSource": {
                        "name": "product#/municipio",
                        "rest": true
                    },
                    "outData": {
                        "NMMUNICIPIO": "NMMUNICIPIO",
                        "CDMUNICIPIO": "CDMUNICIPIO"
                    },
                    "dataSourceFilter": [],
                    "showSelectAs": "grid",
                    "fields": [
                        {
                            "name": "NMMUNICIPIO",
                            "label": "Município",
                            "isVisible": true,
                            "showOnList": true,
                            "template": "field/text-edit.html"
                        }
                    ],
                    "events": [
                        {
                            "name": "FieldOnChange",
                            "code": "ABAEnderecoParceiroController.checkPlaceChange(args.owner.field);"
                        },
                        {
                            "id": "12932085961632664968982",
                            "name": "FieldOnAdd",
                            "code": "crm00040_Lead_Controller.criarNovoMunicipio(args.owner.field.widget)"
                        },
                        {
                            "name": "FieldOnClick",
                            "code": "ABAEnderecoParceiroController.addressFieldOnClick(args.owner.field);"
                        }
                    ],
                    "filterProperties": {
                        "events": [],
                        "validations": {},
                        "readOnly": false
                    }
                },
                {
                    "name": "NMBAIRRO",
                    "label": "Bairro",
                    "template": "field/select.html",
                    "class": "6",
                    "isVisible": true,
                    "showOnList": false,
                    "showOnView": true,
                    "showOnForm": true,
                    "readOnly": true,
                    "defaultValue": "",
                    "validations": {
                        "required": true
                    },
                    "showAddButtonAs": "button",
                    "addButtonText": "Adicionar Novo Município",
                    "valueField": "CDBAIRRO",
                    "descriptionField": "NMBAIRRO",
                    "dataSource": {
                        "name": "product#/bairro",
                        "rest": true
                    },
                    "outData": {
                        "CDBAIRRO": "CDBAIRRO",
                        "NMBAIRRO": "NMBAIRRO"
                    },
                    "dataSourceFilter": [],
                    "showSelectAs": "grid",
                    "fields": [
                        {
                            "name": "NMBAIRRO",
                            "label": "Bairro",
                            "isVisible": true,
                            "showOnList": true,
                            "template": "field/text-edit.html"
                        }
                    ],
                    "events": [
                        {
                            "name": "FieldOnClick",
                            "code": "ABAEnderecoParceiroController.addressFieldOnClick(args.owner.field);"
                        },
                        {
                            "id": "1722319758387363832512",
                            "name": "FieldOnAdd",
                            "code": "crm00040_Lead_Controller.criarNovoBairro(args.owner.field.widget)"
                        }
                    ],
                    "filterProperties": {
                        "events": [],
                        "validations": {},
                        "readOnly": false
                    }
                },
                {
                    "name": "DSREFERENCIAENDE",
                    "label": "Referência",
                    "template": "field/textarea.html",
                    "class": "12",
                    "isVisible": true,
                    "showOnList": false,
                    "showOnView": true,
                    "showOnForm": true,
                    "textRows": 5,
                    "validations": {},
                    "events": []
                },
                {
                    "name": "DSLATITUDE",
                    "label": "Latitude",
                    "template": "field/text-edit.html",
                    "fieldGroup": "geolocalizacao",
                    "help": "Estes campos são necessários para que o registro apareça na tela 'Geolocalizacao'",
                    "class": "6",
                    "isVisible": true,
                    "showOnList": false,
                    "showOnView": true,
                    "showOnForm": true,
                    "textRows": 5,
                    "validations": {},
                    "events": []
                },
                {
                    "name": "DSLONGITUDE",
                    "label": "Longitude",
                    "template": "field/text-edit.html",
                    "fieldGroup": "geolocalizacao",
                    "help": "Estes campos são necessários para que o registro apareça na tela 'Geolocalizacao'",
                    "class": "6",
                    "isVisible": true,
                    "showOnList": false,
                    "showOnView": true,
                    "showOnForm": true,
                    "textRows": 5,
                    "validations": {},
                    "events": []
                }
            ],
            "widgets": [
                {
                    "id":"1722319758311315390518",
                    "name": "",
                    "label": "",
                    "template": "widget/form.html",
                    "container": activeWidget.container,
                    "isVisible": false,
                    "showAsTab": false,
                    "dataSource": {},
                    "events": [],
                    "actions": [],
                    "fields": [],
                    "widgets": []
                }
            ]
        };    
        
        addressWidget = metaDataFactory.widgetFactory(addressWidget);
        CRM_Utilities.factoryWidgetActions(addressWidget);
        
        return addressWidget;
        
    };
    
    /**
     * Executa as funções relacionadas a filtro da
     * widget de endereço.
     * 
     * @param {object} widgte widget de endereço
     * ("WDGT_ENDERECO").
    */
    this.setAddressWidgetFilters = function(widget){
        
        this.setAddressWidgetFilterBySelectedPartner(widget);
        
        widget.reload();
        
    };
    
    /**
     * Define o filtro da widget de enderço pelo parceiro selecionado.
     *
     * @param {object} widget widget de endereço.
    */
    this.setAddressWidgetFilterBySelectedPartner = function(widget){
        
        var parentWidgetCurrentRow = widget.getParent().currentRow;
        
        var widgetFilter = widget.dataSourceFilter;
        var nrparcnegocioFilterIndex = CRM_Utilities.getObjectPositionInArrayByName(widgetFilter, "NRPARCNEGOCIO");
        
        if(widgetFilter[nrparcnegocioFilterIndex].value !== parentWidgetCurrentRow.NRPARCNEGOCIO){
            widgetFilter[nrparcnegocioFilterIndex].value = parentWidgetCurrentRow.NRPARCNEGOCIO;
        }    
        
    };
    
    /**
     * Define o filtro do campo 'Tipo do Endereço' pelo
     * parceiro selecionado.
     * Definindo o filtro, o usuário consegue cadastrar
     * apenas um endereço por tipo, lembrando que se o usuário
     * for editar um endereço, o tipo previamente cadastra
     * é exibido, este é o motivo do filtro do campo CDTIPOENDERECO.
     * 
     * @param {object} widget widget de endereços
     * ("WDGT_ENDERECOS").
    */
    this.setAddressTypeFieldFilter = function(widget){
        
        var parentWidgetCurrentRow = widget.getParent().currentRow;
        var currentRow = widget.currentRow;
        
        var relatedPartnerCode = parentWidgetCurrentRow.NRPARCNEGOCIO;
        var relatedOrgCode = parentWidgetCurrentRow.NRORG;
        var currentAddressType = currentRow.CDTIPOENDERECO;
        
        var addressTypeField = widget.getField("NMTIPOENDERECO");
        var addressTypeFieldFilter = addressTypeField.dataSourceFilter;
        
        var nrparcnegocioFilterIndex = CRM_Utilities.getObjectPositionInArrayByName(addressTypeFieldFilter, "P_NRPARCNEGOCIO");
        var nrorgFilterIndex = CRM_Utilities.getObjectPositionInArrayByName(addressTypeFieldFilter, "P_NRORG");
        var cdtipoenderecoFilterIndex = CRM_Utilities.getObjectPositionInArrayByName(addressTypeFieldFilter, "P_CDTIPOENDERECO");
        
        
        addressTypeFieldFilter[nrparcnegocioFilterIndex].value = relatedPartnerCode;
        addressTypeFieldFilter[nrorgFilterIndex].value = relatedOrgCode;
        addressTypeFieldFilter[cdtipoenderecoFilterIndex].value = currentAddressType ? currentAddressType : "-1";
        
        addressTypeField.reload();    
        
        
    };
    
    /**
     * Verifica se o usuário modificou a localidade, em caso,
     * selecionada.
     * 
     * @param {object} field campo a ser verificado.
    */
    this.SELECTEDPLACES = {COUNTRY: "", STATE: "", CITY: ""};
    this.checkPlaceChange = function(field){
        
        var widget = field.widget;
        var currentRow = widget.currentRow;
        
        switch(field.name){
            
            case "NMPAIS":
                
                if(currentRow.NMPAIS !== this.SELECTEDPLACES.COUNTRY){
                    
                    this.cleanSelectedPlacesByPlaceHierarchy(widget, "STATE");
                    this.SELECTEDPLACES.COUNTRY = currentRow.NMPAIS;
                    
                }
                
                break;
                
            case "NMESTADO":
                
                if(currentRow.NMESTADO !== this.SELECTEDPLACES.STATE){
                    
                    this.cleanSelectedPlacesByPlaceHierarchy(widget, "CITY");
                    this.SELECTEDPLACES.STATE = currentRow.NMESTADO;
                    
                }
                
                break;
                
            case "NMMUNICIPIO":
                
                if(currentRow.NMMUNICIPIO !== this.SELECTEDPLACES.COUNTRY){
                    
                    this.cleanSelectedPlacesByPlaceHierarchy(widget, "NEIGHBORHOOD");
                    this.SELECTEDPLACES.CITY = currentRow.NMMUNICIPIO;
                    
                }
                
                break;
            
        }
        
        this.unlockPlaceFieldsByFilledPlaces(widget);
        
    };
    
    /**
     * Atualiza os locais preenchidos na variável
     * relacionada a locais selecionados
     * (SELECTEDPLACES).
     * 
     * @param {object} widget widget de endereços
     * ("WDGT_ENDERECOS").
    */
    this.checkPlacesFilled = function(widget){
        
        var currentRow = widget.currentRow;
        
        this.SELECTEDPLACES.COUNTRY = currentRow.NMPAIS;
        this.SELECTEDPLACES.STATE = currentRow.NMESTADO;
        this.SELECTEDPLACES.CITY = currentRow.NMMUNICIPIO;
        
    };
    
    /**
     * Limpa os campo de endereço a partir do local selecionado
     * (olhando a hierarquia de localidade).
     * EX.:: Se o parâmetro 'referencePlace' for 'STATE', o procedimento
     * irá limpar todos os campo a partir de estado (incluindo o estado)
     * na currentRow.
     * 
     * @param {object} widget widget de endereço ('WDGT_ENDERECO').
     * 
     * @param {string} referencePlace localidade referência para a função executar
     * a limpeza.
    */
    this.PLACEHIERARCHY = {
        COUNTRY: ["NMPAIS", "CDPAIS"], 
        STATE: ["NMESTADO", "SGESTADO"], 
        CITY: ["NMMUNICIPIO", "CDMUNICIPIO"], 
        NEIGHBORHOOD: ["NMBAIRRO", "CDBAIRRO"]
    };
    this.cleanSelectedPlacesByPlaceHierarchy = function(widget, referencePlace){
        
        var currentRow = widget.currentRow;
        var toClean = false;
        
        for(var place in this.PLACEHIERARCHY){
            
            if(place === referencePlace){
                toClean = true;
            }
            
            if(toClean){
                
                this.PLACEHIERARCHY[place].forEach(function(placeField){
                    currentRow[placeField] = "";
                });
                
            }
            
        }
        
    };
    
    /**
     * Libera os campos de endereço de acordo com o preenchimento
     * dos mesmos, pois através da regra de negócio do sistema, o
     * usuário deve preencher um endereço de acordo com a hierarquia
     * de localidade.
     * 
     * @param {object} widget widget de endereços("WDGT_ENDEREÇOS").
    */
    this.unlockPlaceFieldsByFilledPlaces = function(widget){
        
        var arrayOfPlaces = [
            widget.getField("NMPAIS"),
            widget.getField("NMESTADO"),
            widget.getField("NMMUNICIPIO"),
            widget.getField("NMBAIRRO")
        ];
        var currentRow = widget.currentRow;
        
        arrayOfPlaces.forEach(function(place, index){
            
            if(place.name !== "NMBAIRRO"){
                
                if(Boolean(currentRow[place.name])){
                    CRM_Utilities.setFieldReadOnlyProperty(arrayOfPlaces[index + 1], false);
                } else {
                    CRM_Utilities.setFieldReadOnlyProperty(arrayOfPlaces[index + 1], true);
                }
                
            } 
            
        });
        
    };
    
    /**
     * Dispara os eventos necessários ao clicar
     * em um campo de endereço.
     * 
     * @param {object} field campo selecionado.
    */
    this.addressFieldOnClick = function(field){
        
        CRM_Utilities.setPlaceFilterByPlaceOrigin(field.widget, field);
        CRM_Utilities.setDefaultOrgFilterAndOperatorOrgFilter(field);
        
        field.reload();
        
    };
    
    /**
     * Preenche os campos de endereço pelo CEP
     * digitado.
     * 
     * @param {object} widget widget de endereços
     * ("WDGT_ENDEREÇOS").
    */
    this.fillAddressByCep = function (widget) {

        var currentRow = widget.currentRow;
        var cep = currentRow.NRCEPENDERECO;
        const nrCepEndereco = widget.currentRow.NRCEPENDERECO;
        const cepSemMascara = nrCepEndereco.replace(/\D/g, '');

        if (cep && cepSemMascara.length === 8) {

            CRM_Utilities.buscarCep(cep).then(function (response) {

                var foundAddress = response.dataset.ENDERECO;

                currentRow.NMPAIS = "Brasil";
                currentRow.CDPAIS = "0055";
                currentRow.NMESTADO = foundAddress.NMESTADO;
                currentRow.SGESTADO = foundAddress.SGESTADO;
                currentRow.NMMUNICIPIO = foundAddress.NMMUNICIPIO;
                currentRow.CDMUNICIPIO = foundAddress.CDMUNICIPIO;
                currentRow.NMBAIRRO = foundAddress.NMBAIRRO;
                currentRow.CDBAIRRO = foundAddress.CDBAIRRO;
                currentRow.DSENDERECO = foundAddress.DSENDERECO;

                this.unlockPlaceFieldsByFilledPlaces(widget);

            }.bind(this))
                .catch(function (error) {
                    ScreenService.showMessage(error.error);
                });

        }

    };
    
    /**
     * Habilita o ambiente de inserção
     * de um endereço.
     * 
     * @param {object} widget widget de 
     * endereços ("WDGT_ENDERECOS").
    */
    this.addAddress = function(widget){
        var nrParcNegocio = widget.parent.currentRow.NRPARCNEGOCIO;
        
        this.checkIfCanEditOrEvolParc(nrParcNegocio).then(function(canEdit) {
            if (canEdit) {
                General.widgetNewRow(widget);
                
                this.unlockPlaceFieldsByFilledPlaces(widget);
                this.setAddressDefaultValues(widget);
            } else {
                this.showMessageCantEditParc();
            }
        }.bind(this));
    };
    
    /**
     * Habilita o ambiente de edição de 
     * um endereço.
     * 
     * @param {object} widget widget de 
     * endereços ("WDGT_ENDERECOS").
    */
    this.editAddress = function(widget){
        var nrParcNegocio = widget.currentRow.NRPARCNEGOCIO;
        
        this.checkIfCanEditOrEvolParc(nrParcNegocio).then(function(canEdit) {
            if (canEdit) {
                General.widgetEdit(widget);
                
                this.unlockPlaceFieldsByFilledPlaces(widget);
                this.checkPlacesFilled(widget);
            } else {
                this.showMessageCantEditParc();
            }
        }.bind(this));
    };
    
    /**
     * Define os valores padrões de um endereço.
     * 
     * @param {object} widget widget de endereço.
    */
    this.setAddressDefaultValues = function(widget){
        
        var parentWidgetCurrentRow = widget.getParent().currentRow;
        var currentRow = widget.currentRow;
        
        currentRow.NRPARCNEGOCIO = parentWidgetCurrentRow.NRPARCNEGOCIO;
        
    };
    
    this.searchLocation = function() {
        var nrParcNegocio = templateManager.container.getActiveWidget().currentRow.NRPARCNEGOCIO;
    
        this.checkIfCanEditOrEvolParc(nrParcNegocio).then(function(canEdit) {
            if (canEdit) {
                ScreenService.confirmMessage('Os campos de Latitude e Longitude serão atualizados de acordo com os dados dos campos "Endereço", "Número" e "Estado", deseja continuar? ', 'CONFIRMATION', function(success) {
                    var widget = templateManager.container.getActiveWidget();
                    var currentRow = widget.currentRow;
                    var geocoder = new google.maps.Geocoder();
                    var endereco = currentRow.DSENDERECO + ', ' + currentRow.NRIMOVELENDERECO + ', ' + currentRow.SGESTADO;
    
                    geocoder.geocode({ 'address': endereco }, function(results, status) {
                        if (status == google.maps.GeocoderStatus.OK) {
                            var latitude = results[0].geometry.location.lat();
                            var longitude = results[0].geometry.location.lng();
                            currentRow.DSLATITUDE = latitude;
                            currentRow.DSLONGITUDE = longitude;
                            restEngine.post('/atualizaLatLong', {requestType: "DataSet", dataset: [currentRow]}).then(function() {
                                widget.reload();
                            });
                        } else if (status == 'OVER_QUERY_LIMIT') {
                            ScreenService.showMessage("Limite de pesquisa por latitude e longitude por tempo ou diário excedido! Espere alguns minutos e tente novamente ou tente novamente em outro dia.", "ERROR");
                        }
                    });
                }.bind(this));
            } else {
                this.showMessageCantEditParc();
            }
        }.bind(this));
    };
    
    this.deleteAddress = function(widget) {
        var nrParcNegocio = widget.currentRow.NRPARCNEGOCIO;
    
        this.checkIfCanEditOrEvolParc(nrParcNegocio).then(function(canEdit) {
            if (canEdit) {
                General.deleteCrud(widget);
            } else {
                this.showMessageCantEditParc();
            }
        }.bind(this));
    };
    
    this.showMessageCantEditParc = function() {
        var message = "Ação bloqueada!<br>" +
            "Somente o operador responsável pelo prospect e seus assistentes podem " +
            "editá-lo/evoluí-lo.";
    
        ScreenService.showMessage(message);
    };
    
    this.checkIfCanEditOrEvolParc = function(nrParcNegocio) {
        var restRow = {
            NRPARCNEGOCIO: nrParcNegocio,
            CRM_NRORGTRAB: getLocalVar('CRM_NRORGTRABALHO')
        };
        
        var params = {
            requestType: 'Row',
            row: restRow
        };
        
        return restEngine.post('/checkIfCanEditOrEvolParc', params)
            .then(function(response) {
                return response.dataset.DADOS.CAN_EDIT;
            })
            .catch(function(error) {
                return false;
            });
    };
    
}

Configuration(function(ContextRegister) {
    ContextRegister.register('ABAEnderecoParceiroController', ABAEnderecoParceiroController);
});